#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
import datetime
import io
import tarfile

import attr
import bs4

from cryptodatahub.common.entity import Entity
from cryptodatahub.common.stores import (
    CertificateTrustConstraint,
    RootCertificate,
    RootCertificateParams,
    RootCertificateTrustStoreConstraint,
    RootCertificateTrustConstraintAction,
)
from cryptodatahub.common.types import Base64Data, CryptoDataParamsBase

from updaters.common import CertificatePemFetcher, FetcherBase, FetcherCsvBase, HttpFetcher, UpdaterBase


@attr.s
class RootCertificateStore(CryptoDataParamsBase):
    certificates = attr.ib(
        default={},
        validator=attr.validators.deep_mapping(
            key_validator=attr.validators.instance_of(Base64Data),
            value_validator=attr.validators.instance_of(RootCertificateTrustConstraintAction)
        )
    )


class FetcherRootCertificateStoreGoogle(FetcherBase):
    @classmethod
    def _get_current_data(cls):
        data = HttpFetcher()(
            'https://android.googlesource.com/platform/system/ca-certificates/+archive/refs/heads/master/files.tar.gz'
        )
        with tarfile.open(fileobj=io.BytesIO(data), mode='r') as tar:
            for member in tar.getmembers():
                yield tar.extractfile(member).read().decode('ascii')

    @classmethod
    def _transform_data(cls, current_data):
        certificates = {}
        for root_certificate_pem in current_data:
            root_certificate_pem_lines = root_certificate_pem.splitlines()
            root_certificate_pem_lines = root_certificate_pem_lines[
                :root_certificate_pem_lines.index('-----END CERTIFICATE-----') + 1
            ]

            certificates[tuple(root_certificate_pem_lines)] = tuple()

        return certificates


class FetcherRootCertificateStoreApple(FetcherBase):
    FIELDS = (
        'Certificate name',
        'Issued by',
        'Type',
        'Key size',
        'Sig alg',
        'Serial number',
        'Expires',
        'EV policy',
        'Fingerprint (SHA-256)',
    )

    @classmethod
    def _get_current_data(cls):
        response = HttpFetcher()('https://support.apple.com/en-us/HT213080')
        page = bs4.BeautifulSoup(response.decode('utf-8'), 'html.parser')
        trusted_cas = page.find("h2", {"id": "trusted"})
        return [
            dict(zip(cls.FIELDS, map(lambda value: value.text.strip(), row.find_all('td'))))
            for row in trusted_cas.find_next('tbody').find_all('tr')[1:]
        ]

    @classmethod
    def _transform_data(cls, current_data):
        certificate_pem_fetcher = CertificatePemFetcher()
        certificates = {}
        for row in current_data:
            sha2_256_fingerprint = row['Fingerprint (SHA-256)'].replace(' ', '')
            root_certificate_pem = certificate_pem_fetcher(sha2_256_fingerprint)
            certificates[tuple(root_certificate_pem.splitlines())] = ()

        return certificates


class FetcherRootCertificateStoreMicrosoft(FetcherCsvBase):
    CSV_FIELDS = (
        'Microsoft Status',
        'CA Owner',
        'CA Common Name or Certificate Name',
        'SHA-1 Fingerprint',
        'SHA-256 Fingerprint',
        'Microsoft EKUs',
        'Valid From [GMT]',
        'Valid To [GMT]',
        'Public Key Algorithm',
        'Signature Hash Algorithm',
    )

    @classmethod
    def _get_csv_url(cls):
        return 'https://ccadb-public.secure.force.com/microsoft/IncludedCACertificateReportForMSFTCSV'

    @classmethod
    def _get_csv_fields(cls):
        return cls.CSV_FIELDS

    @classmethod
    def _transform_data(cls, current_data):
        certificate_pem_fetcher = CertificatePemFetcher()
        certificates = {}

        for row in current_data:
            status = row['Microsoft Status']

            if status == 'Disabled':
                continue

            constraints = []
            if status == 'NotBefore':
                constraint = CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    date=datetime.datetime.strptime(row['Valid From [GMT]'], '%Y %b %d'),
                )
                constraints.append(constraint)

            sha2_256_fingerprint = row['SHA-256 Fingerprint']
            root_certificate_pem = certificate_pem_fetcher(sha2_256_fingerprint)

            certificates[tuple(root_certificate_pem.splitlines())] = tuple(constraints)

        return certificates


class FetcherRootCertificateStoreMozilla(FetcherCsvBase):
    CSV_FIELDS = ('PEM', 'Distrust for TLS After Date', 'Mozilla Applied Constraints')

    @classmethod
    def _get_csv_url(cls):
        return \
            'https://ccadb.my.salesforce-sites.com/mozilla/IncludedRootsDistrustTLSSSLPEMCSV?TrustBitsInclude=Websites'

    @classmethod
    def _get_csv_fields(cls):
        return cls.CSV_FIELDS

    @classmethod
    def _transform_data(cls, current_data):
        certificates = {}

        for row in current_data:
            constraints = []

            distrust_after = row['Distrust for TLS After Date']
            if distrust_after:
                constraint = CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    date=datetime.datetime.strptime(distrust_after, '%Y.%m.%d'),
                )
                constraints.append(constraint)

            domain = row['Mozilla Applied Constraints']
            if domain:
                constraint = CertificateTrustConstraint(
                    action=RootCertificateTrustConstraintAction.DISTRUST,
                    domains=[domain],
                )
                constraints.append(constraint)

            root_certificate_pem_lines = tuple(row['PEM'].strip('\'').splitlines())
            certificates[root_certificate_pem_lines] = tuple(constraints)

        return certificates


class FetcherRootCertificateStore(FetcherBase):
    _ROOT_CERTIFICATE_STORE_UPDATERS = collections.OrderedDict([
        (Entity.MOZILLA, FetcherRootCertificateStoreMozilla),
        (Entity.GOOGLE, FetcherRootCertificateStoreGoogle),
        (Entity.MICROSOFT, FetcherRootCertificateStoreMicrosoft),
        (Entity.APPLE, FetcherRootCertificateStoreApple),
    ])

    @classmethod
    def _get_current_data(cls):
        return collections.OrderedDict([
            (store_owner, store_fetcher_class.from_current_data())
            for store_owner, store_fetcher_class in cls._ROOT_CERTIFICATE_STORE_UPDATERS.items()
        ])

    @classmethod
    def _transform_data(cls, current_data):
        root_certificates = {}

        for store_owner, root_store in current_data.items():
            for certificate_data, constraints in root_store.parsed_data.items():
                if certificate_data in root_certificates:
                    merged_constraints = root_certificates[certificate_data]
                else:
                    merged_constraints = []
                    root_certificates[certificate_data] = merged_constraints

                merged_constraints.append(RootCertificateTrustStoreConstraint(store_owner, constraints))

        return [
            RootCertificateParams(*item)
            for item in root_certificates.items()
        ]


class UpdaterRootCertificateTrustStore(UpdaterBase):
    def __init__(self):
        super().__init__(  # pragma: no cover
            fetcher_class=FetcherRootCertificateStore,
            enum_class=RootCertificate,
            enum_param_class=RootCertificateParams,
        )


def main():
    UpdaterRootCertificateTrustStore()()  # pragma: no cover


if __name__ == '__main__':
    main()  # pragma: no cover



# -*- coding: utf-8 -*-

import unittest

from test.common.classes import TestClasses, TestGradeableSimple

from cryptodatahub.common.grade import (
    AttackNamed,
    AttackType,
    Grade,
    Gradeable,
    GradeableComplex,
    GradeableVulnerabilities,
    Vulnerability,
)


class TestGradeable(unittest.TestCase):
    _VULNERABILITY_INTEGER_FACTORIZATION_INSECURE = Vulnerability(
        attack_type=AttackType.INTEGER_FACTORIZATION,
        grade=Grade.INSECURE,
        named=None
    )
    _VULNERABILITY_INTEGER_FACTORIZATION_WEAK = Vulnerability(
        attack_type=AttackType.INTEGER_FACTORIZATION,
        grade=Grade.WEAK,
        named=None
    )

    def test_min_grade(self):
        self.assertEqual(None, Gradeable.get_min_grade(None))
        self.assertEqual(None, Gradeable.get_min_grade([None]))
        self.assertEqual(None, Gradeable.get_min_grade([None, [None]]))

        self.assertEqual(Grade.SECURE, Gradeable.get_min_grade([]))
        self.assertEqual(Grade.SECURE, Gradeable.get_min_grade([[]]))

        self.assertEqual(TestGradeableSimple().grade, TestGradeableSimple().min_grade)
        self.assertEqual(TestGradeableSimple().grade.value.name, str(TestGradeableSimple()))

        gradeable_single = GradeableVulnerabilities([
            self._VULNERABILITY_INTEGER_FACTORIZATION_WEAK,
            self._VULNERABILITY_INTEGER_FACTORIZATION_INSECURE,
        ])
        self.assertEqual(Grade.INSECURE, gradeable_single.min_grade)
        self.assertEqual(Grade.INSECURE, Gradeable.get_min_grade([gradeable_single]))
        self.assertEqual(Grade.INSECURE, Gradeable.get_min_grade([
            self._VULNERABILITY_INTEGER_FACTORIZATION_WEAK,
            None,
            self._VULNERABILITY_INTEGER_FACTORIZATION_INSECURE,
        ]))

        gradeable_multiple = GradeableComplex()
        object.__setattr__(gradeable_multiple, 'gradeables', [
            self._VULNERABILITY_INTEGER_FACTORIZATION_WEAK,
            self._VULNERABILITY_INTEGER_FACTORIZATION_INSECURE,
        ])
        self.assertEqual(Grade.INSECURE, gradeable_multiple.min_grade)
        self.assertEqual(Grade.INSECURE, Gradeable.get_min_grade([gradeable_multiple]))
        self.assertEqual(Grade.INSECURE, Gradeable.get_min_grade([
            self._VULNERABILITY_INTEGER_FACTORIZATION_WEAK,
            None,
            self._VULNERABILITY_INTEGER_FACTORIZATION_INSECURE,
        ]))


class TestGradeTypeParams(unittest.TestCase):
    def test_eq(self):
        self.assertEqual(Grade.SECURE.value, Grade.SECURE.value)
        self.assertNotEqual(Grade.SECURE.value, Grade.INSECURE.value)

    def test_order(self):
        self.assertEqual(
            list(sorted(Grade, key=lambda grade: grade.value)),
            [Grade.INSECURE, Grade.WEAK, Grade.DEPRECATED, Grade.SECURE]
        )


class TestAttackNamed(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return AttackNamed


class TestAttackType(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return AttackType



# -*- coding: utf-8 -*-

import abc
import collections
import csv
import io


import attr

from cryptodatahub.common.utils import HttpFetcher
from cryptodatahub.common.stores import RootCertificate


@attr.s(frozen=True)
class CertificatePemFetcher():
    http_fetcher = attr.ib(
        init=False,
        default=HttpFetcher(connect_timeout=5, read_timeout=30, retry=10),
        validator=attr.validators.instance_of(HttpFetcher),
    )

    def __call__(self, sha2_256_fingerprint):
        try:
            return RootCertificate.get_item_by_sha2_256_fingerprint(
                sha2_256_fingerprint
            ).value.certificate.pem
        except KeyError:
            data = self.http_fetcher(f'https://crt.sh/?d={sha2_256_fingerprint}')
            return data.decode('utf-8').strip()


@attr.s
class FetcherBase():
    parsed_data = attr.ib(validator=attr.validators.instance_of(collections.abc.Iterable))

    @classmethod
    @abc.abstractmethod
    def _get_current_data(cls):
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def _transform_data(cls, current_data):
        raise NotImplementedError()

    @classmethod
    def from_current_data(cls):
        current_data = cls._get_current_data()
        transformed_data = cls._transform_data(current_data)

        return cls(transformed_data)


@attr.s
class FetcherCsvBase(FetcherBase):
    @classmethod
    @abc.abstractmethod
    def _get_csv_url(cls):
        raise NotImplementedError()

    @classmethod
    @abc.abstractmethod
    def _get_csv_fields(cls):
        raise NotImplementedError()

    @classmethod
    def _get_fetcher(cls):
        return HttpFetcher()

    @classmethod
    def _get_current_data(cls):
        data = cls._get_fetcher()(cls._get_csv_url())

        csv_reader = csv.DictReader(
            io.StringIO(data.decode('utf-8')),
            fieldnames=cls._get_csv_fields(),
        )

        sample = data[:4096].decode('utf-8')
        if csv.Sniffer().has_header(sample):
            next(csv_reader, None)

        return csv_reader

    @classmethod
    @abc.abstractmethod
    def _transform_data(cls, current_data):
        raise NotImplementedError()


@attr.s
class UpdaterBase():
    fetcher_class = attr.ib(validator=attr.validators.instance_of(type))
    enum_class = attr.ib(validator=attr.validators.instance_of(type))
    enum_param_class = attr.ib(validator=attr.validators.instance_of(type))

    def _has_item_changed(self, enum_item_name, enum_item_value):
        try:
            return self.enum_class[enum_item_name].value != enum_item_value
        except KeyError:
            return True

    def _has_data_changed(self, current_data_items_by_name):
        return any(map(
            lambda item: self._has_item_changed(*item),
            current_data_items_by_name.items()
        ))

    def __call__(self):
        current_data = self.fetcher_class.from_current_data()
        current_data_items_by_name = {
            fetched_data_item.identifier: fetched_data_item
            for fetched_data_item in current_data.parsed_data
        }
        if self._has_data_changed(current_data_items_by_name):
            self.enum_class.set_json(self.enum_param_class, collections.OrderedDict([
                (identifier, current_data_items_by_name[identifier]._asdict())
                for identifier in sorted(current_data_items_by_name.keys())
            ]))



# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.common.algorithm import Authentication, BlockCipher, BlockCipherMode, KeyExchange, MAC

from cryptodatahub.tls.algorithm import (
    SslCipherKind,
    TlsCertificateCompressionAlgorithm,
    TlsCipherSuite,
    TlsCipherSuiteExtension,
    TlsCompressionMethod,
    TlsECPointFormat,
    TlsExtensionType,
    TlsGreaseOneByte,
    TlsGreaseTwoByte,
    TlsNamedCurve,
    TlsNextProtocolName,
    TlsProtocolName,
    TlsPskKeyExchangeMode,
    TlsSignatureAndHashAlgorithm,
    TlsTokenBindingParamater,
)


class TestSslCipherKind(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return SslCipherKind


class TestTlsCertificateCompressionAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsCertificateCompressionAlgorithm


class TestTlsCipherSuite(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsCipherSuite

    def test_str(self):
        self.assertEqual(
            str(TlsCipherSuite.TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256.value),
            'TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256 (ECDH-RSA-AES128-SHA256)'
        )
        self.assertEqual(
            str(TlsCipherSuite.TLS_ECDH_ECDSA_WITH_DES_CBC_SHA.value),
            'TLS_ECDH_ECDSA_WITH_DES_CBC_SHA'
        )
        self.assertEqual(
            str(TlsCipherSuite.OLD_TLS_ECDH_ECDSA_WITH_NULL_SHA.value),
            'OLD_TLS_ECDH_ECDSA_WITH_NULL_SHA'
        )

    def test_gradeables(self):
        self.assertEqual(
            TlsCipherSuite.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256.value.gradeables, [
                KeyExchange.ECDHE.value,
                Authentication.RSA.value,
                BlockCipher.AES_128.value,
                BlockCipherMode.GCM.value,
                MAC.SHA2_256.value,
            ]
        )
        self.assertEqual(
            TlsCipherSuite.TLS_RSA_EXPORT_WITH_RC4_40_MD5.value.gradeables, [
                KeyExchange.RSA.value,
                Authentication.RSA.value,
                BlockCipher.RC4_40.value,
                MAC.MD5.value,
            ]
        )


class TestTlsCipherSuiteExtension(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsCipherSuiteExtension


class TestTlsCompressionMethod(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsCompressionMethod


class TestTlsECPointFormat(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsECPointFormat


class TestTlsExtensionType(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsExtensionType


class TestTlsGreaseOneByte(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsGreaseOneByte


class TestTlsGreaseTwoByte(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsGreaseTwoByte


class TestTlsNamedCurve(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsNamedCurve

    def test_str(self):
        self.assertEqual(str(TlsNamedCurve.X25519.value), 'Curve25519')


class TestTlsNextProtocolName(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return TlsNextProtocolName


class TestTlsProtocolName(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return TlsProtocolName


class TestTlsPskKeyExchangeMode(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsPskKeyExchangeMode


class TestTlsSignatureAndHashAlgorithm(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsSignatureAndHashAlgorithm

    def test_str(self):
        self.assertEqual(str(TlsSignatureAndHashAlgorithm.ANONYMOUS_NONE.value), 'none with no encryption')
        self.assertEqual(str(TlsSignatureAndHashAlgorithm.ANONYMOUS_SHA1.value), 'SHA-1 with no encryption')
        self.assertEqual(str(TlsSignatureAndHashAlgorithm.RSA_SHA1.value), 'SHA-1 with RSA encryption')


class TestTlsTokenBindingParamater(TestClasses.TestJsonCodeNumericBase):
    @classmethod
    def _get_class(cls):
        return TlsTokenBindingParamater



# -*- coding: utf-8 -*-

try:
    import unittest2 as unittest
except ImportError:
    import unittest

from test.common.classes import TestClasses

from cryptodatahub.common.algorithm import NamedGroup
from cryptodatahub.common.exception import InvalidValue
from cryptodatahub.common.parameter import DHParameterNumbers, DHParamWellKnown, ECParamWellKnown


class TestDHParamWellKnownParams(unittest.TestCase):
    def test_eq(self):
        self.assertEqual(DHParameterNumbers(1, 2), DHParameterNumbers(1, 2))
        self.assertEqual(DHParameterNumbers(1, 2, None), DHParameterNumbers(1, 2, None))

        self.assertNotEqual(DHParameterNumbers(1, 2, 3), DHParameterNumbers(1, 2, 4))
        self.assertEqual(DHParameterNumbers(1, 2, None), DHParameterNumbers(1, 2, 3))


class TestDHParamWellKnown(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return DHParamWellKnown

    def test_from_parameter_numbers(self):
        dh_param = DHParamWellKnown.RFC2539_1024_BIT_MODP_GROUP
        self.assertEqual(
            DHParamWellKnown.from_parameter_numbers(dh_param.value.parameter_numbers),
            dh_param
        )

    def test_str(self):
        self.assertEqual(
            str(DHParamWellKnown.RFC2539_768_BIT_MODP_GROUP.value),
            '768-bit MODP Group/Oakley Group 1 (RFC 2409, RFC 2539)'
        )
        self.assertEqual(
            str(DHParamWellKnown.APPLICATION_SERVER_APACHE_VERSION_2_0_18_BIT_1024.value),
            '1024-bit Apache 2.0.18 builtin DH parameter'
        )


class TestECParamWellKnown(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return ECParamWellKnown

    def test_from_named_group(self):
        self.assertEqual(
           ECParamWellKnown.from_named_group(NamedGroup.PRIME256V1),
           ECParamWellKnown.PRIME256V1
        )

        with self.assertRaisesRegex(
                InvalidValue, "NamedGroup.C2ONB239V4.* is not a valid ECParamWellKnown"
        ) as context_manager:
            ECParamWellKnown.from_named_group(NamedGroup.C2ONB239V4)
        self.assertEqual(context_manager.exception.value, NamedGroup.C2ONB239V4)

    def test_from_parameter_numbers(self):
        ec_param = ECParamWellKnown.PRIME256V1
        self.assertEqual(
           ECParamWellKnown.from_parameter_numbers(ec_param.value.parameter_numbers),
           ec_param
        )



# -*- coding: utf-8 -*-

import unittest
from unittest import mock

from test.common.classes import TEST_URL_PREFIX

import urllib3

from cryptodatahub.common.algorithm import Hash
from cryptodatahub.common.utils import (
    HttpFetcher,
    bytes_from_hex_string,
    bytes_to_hex_string,
    hash_bytes,
    name_to_enum_item_name,
)


class TestBytesToHexString(unittest.TestCase):
    def test_separator(self):
        self.assertEqual(bytes_to_hex_string(b''), '')
        self.assertEqual(bytes_to_hex_string(b'\xde\xad\xbe\xef'), 'DEADBEEF')
        self.assertEqual(bytes_to_hex_string(b'\xde\xad\xbe\xef', separator=':'), 'DE:AD:BE:EF')

    def test_lowercase(self):
        self.assertEqual(bytes_to_hex_string(b''), '')
        self.assertEqual(bytes_to_hex_string(b'\xde\xad\xbe\xef'), 'DEADBEEF')
        self.assertEqual(bytes_to_hex_string(b'\xde\xad\xbe\xef', lowercase=True), 'deadbeef')


class TestBytesFromHexString(unittest.TestCase):
    def test_error_odd_length_string(self):
        with self.assertRaises(ValueError) as context_manager:
            bytes_from_hex_string('0d:d')
        self.assertEqual(type(context_manager.exception), ValueError)

    def test_error_non_hex_string(self):
        with self.assertRaises(ValueError) as context_manager:
            bytes_from_hex_string('no:th:ex')
        self.assertEqual(type(context_manager.exception), ValueError)

    def test_separator(self):
        self.assertEqual(bytes_from_hex_string(''), b'')
        self.assertEqual(bytes_from_hex_string('DEADBEEF'), b'\xde\xad\xbe\xef')
        self.assertEqual(bytes_from_hex_string('DE:AD:BE:EF', separator=':'), b'\xde\xad\xbe\xef')


class TestHashBytes(unittest.TestCase):
    def test_error_unknown_hash_algorithm(self):
        with self.assertRaises(NotImplementedError) as context_manager:
            hash_bytes(Hash.SHA3_512, b'')
        self.assertEqual(context_manager.exception.args, (Hash.SHA3_512, ))

    def test_hash(self):
        self.assertEqual(
            hash_bytes(Hash.SHA1, b'abc'),
            b'\xA9\x99\x3E\x36\x47\x06\x81\x6A\xBA\x3E\x25\x71\x78\x50\xC2\x6C\x9C\xD0\xD8\x9D'
        )


class TestNameToEnumItemName(unittest.TestCase):
    def test_convert_simple_name(self):
        self.assertEqual(name_to_enum_item_name('lower'), 'LOWER')

    def test_convert_multipart_name(self):
        self.assertEqual(name_to_enum_item_name('multiple part'), 'MULTIPLE_PART')
        self.assertEqual(name_to_enum_item_name('aplha 123'), 'APLHA_123')
        self.assertEqual(name_to_enum_item_name('m  u  l  t  i  s  p  a  c  e'), 'M_U_L_T_I_S_P_A_C_E')
        self.assertEqual(name_to_enum_item_name('trailing space  '), 'TRAILING_SPACE')
        self.assertEqual(name_to_enum_item_name('apo\'s trophe'), 'APOS_TROPHE')

    def test_convert_i18n_name(self):
        self.assertEqual(name_to_enum_item_name('αβγ'), 'ΑΒΓ')


class TestHttpFetcher(unittest.TestCase):
    @mock.patch.object(urllib3.poolmanager.PoolManager, 'request', side_effect=NotImplementedError)
    def test_error_unhandaled_exception(self, _):
        with self.assertRaises(NotImplementedError):
            HttpFetcher()('http://example.com')

    def test_error_fetch_timeout(self):
        http_fetcher = HttpFetcher(
            connect_timeout=0.000001, read_timeout=0.000001, retry=0
        )
        with self.assertRaises(AttributeError):
            http_fetcher(TEST_URL_PREFIX + 'test.csv')
        with self.assertRaises(AttributeError):
            http_fetcher.get_response_header('Server')
        with self.assertRaises(AttributeError):
            _ = http_fetcher.response_data

    def test_fetch(self):
        http_fetcher = HttpFetcher()
        http_fetcher.fetch(TEST_URL_PREFIX + 'test.html')
        self.assertEqual(http_fetcher.get_response_header('Content-Type'), 'text/plain; charset=utf-8')
        self.assertEqual(http_fetcher.response_data, b'\n'.join([
            b'<!DOCTYPE html>',
            b'<html>',
            b'  <body>',
            b'    Page content',
            b'  </body>',
            b'</html>',
        ]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import collections
import json

import attr
import urllib3

from cryptodatahub.common.entity import Entity
from cryptodatahub.common.stores import CertificateTransparencyLog, CertificateTransparencyLogParams
from cryptodatahub.common.utils import name_to_enum_item_name

from updaters.common import FetcherBase, UpdaterBase


@attr.s
class FetcherCertificateTransparencyLogs(FetcherBase):
    _CT_LOGS_ALL_JSON_URL = 'https://www.gstatic.com/ct/log_list/v3/all_logs_list.json'
    _CT_LOG_OPERATOR_NAME_GOOGLE_NAME = {
        'Up In The Air Consulting': 'Filippo Valsorda',
        'Wang Shengnan': 'GDCA',
    }

    @classmethod
    def _get_current_data(cls):
        http = urllib3.PoolManager()
        response = http.request('GET', cls._CT_LOGS_ALL_JSON_URL, preload_content=False)
        response.release_conn()

        return json.loads(response.data, object_pairs_hook=collections.OrderedDict)

    @classmethod
    def _transform_data(cls, current_data):
        transformed_logs = []

        for operator in current_data['operators']:
            logs = operator.pop('logs')
            for log in logs:
                if 'state' in log:
                    state = log.pop('state')
                    state_type = list(state)[0]
                    state_args = collections.OrderedDict([('state_type', state_type)])
                    state_args.update(state[state_type])
                    state_args.pop('final_tree_head', None)

                    log.update(collections.OrderedDict([('log_state', state_args)]))

                operator_name = operator['name']
                operator_name = cls._CT_LOG_OPERATOR_NAME_GOOGLE_NAME.get(operator_name, operator_name)
                log['operator'] = Entity[name_to_enum_item_name(operator_name)].name

                transformed_logs.append(CertificateTransparencyLogParams(**log))

        return transformed_logs


class UpdaterCertificateTransparencyLogs(UpdaterBase):
    def __init__(self):
        super().__init__(
            FetcherCertificateTransparencyLogs,
            CertificateTransparencyLog,
            CertificateTransparencyLogParams,
        )


def main():
    UpdaterCertificateTransparencyLogs()()  # pragma: no cover


if __name__ == '__main__':
    main()  # pragma: no cover



# -*- coding: utf-8 -*-

import collections
import datetime
import enum
import re

import attr

from cryptodatahub.common.algorithm import Hash
from cryptodatahub.common.entity import Entity, EntityRole
from cryptodatahub.common.exception import InvalidValue
from cryptodatahub.common.key import PublicKeyX509Base
from cryptodatahub.common.types import (
    Base64Data,
    CryptoDataEnumBase,
    CryptoDataParamsBase,
    CryptoDataParamsFetchedBase,
    convert_base64_data,
    convert_datetime,
    convert_dict_to_object,
    convert_enum,
    convert_iterable,
)
from cryptodatahub.common.utils import bytes_to_hex_string, name_to_enum_item_name


@attr.s(frozen=True)
class CertificateTransparencyOperator(CryptoDataParamsBase):
    name = attr.ib(validator=attr.validators.instance_of(str))
    email = attr.ib(
        default=(),
        validator=attr.validators.deep_iterable(attr.validators.instance_of(str))
    )


CertificateTransparencyLogType = enum.Enum('CertificateTransparencyLogType', 'TEST PROD')
CertificateTransparencyLogStateType = enum.Enum(
    'CertificateTransparencyLogStateType', 'PENDING QUALIFIED USABLE READONLY RETIRED REJECTED'
)


class CertificateTransparencyLogDateTimeBase(CryptoDataParamsBase):
    @classmethod
    def _asdict_serializer(cls, _, __, value):
        if isinstance(value, datetime.datetime):
            return value.strftime("%Y-%m-%dT%H:%M:%SZ")

        return super()._asdict_serializer(_, __, value)


@attr.s(frozen=True)
class CertificateTransparencyLogState(CertificateTransparencyLogDateTimeBase):
    state_type = attr.ib(
        converter=convert_enum(CertificateTransparencyLogStateType),
        validator=attr.validators.optional(attr.validators.instance_of(CertificateTransparencyLogStateType))
    )
    timestamp = attr.ib(
        converter=convert_datetime(),
        validator=attr.validators.instance_of(datetime.datetime),
    )


@attr.s(frozen=True)
class CertificateTransparencyLogTemporalInterval(CertificateTransparencyLogDateTimeBase):
    start_inclusive = attr.ib(
        converter=convert_datetime(),
        validator=attr.validators.instance_of(datetime.datetime),
    )
    end_exclusive = attr.ib(
        converter=convert_datetime(),
        validator=attr.validators.instance_of(datetime.datetime),
    )


@attr.s(frozen=True)
class CertificateTransparencyLogParamsBase(CryptoDataParamsBase):
    log_id = attr.ib(
        converter=convert_base64_data(),
        validator=attr.validators.instance_of(Base64Data),
        metadata={'human_readable_name': 'ID'},
    )


class CertificateTransparencyLogUnknown(CertificateTransparencyLogParamsBase):
    def __str__(self):
        return str(self.log_id)


@attr.s(frozen=True)
class CertificateTransparencyLogParams(  # pylint: disable=too-many-instance-attributes
        CertificateTransparencyLogParamsBase
):
    operator = attr.ib(
        converter=convert_enum(Entity),
        validator=attr.validators.instance_of(Entity),
        metadata={'human_friendly': False},
    )
    key = attr.ib(
        converter=convert_base64_data(),
        validator=attr.validators.instance_of(Base64Data),
        metadata={'human_friendly': False},
    )
    url = attr.ib(
        validator=attr.validators.instance_of(str),
        metadata={'human_readable_name': 'URL', 'human_friendly': False},
    )
    mmd = attr.ib(
        validator=attr.validators.and_(attr.validators.instance_of(int), attr.validators.ge(1)),
        metadata={'human_readable_name': 'Maximum Merge Delay', 'human_friendly': False},
    )
    description = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(str)),
    )
    dns = attr.ib(
        default=None,
        validator=attr.validators.optional(attr.validators.instance_of(str)),
        metadata={'human_friendly': False},
    )
    temporal_interval = attr.ib(
        default=None,
        converter=convert_dict_to_object(CertificateTransparencyLogTemporalInterval),
        validator=attr.validators.optional(attr.validators.instance_of(CertificateTransparencyLogTemporalInterval)),
        metadata={'human_readable_name': 'DNS', 'human_friendly': False},
    )
    log_type = attr.ib(
        default=None,
        converter=convert_enum(CertificateTransparencyLogType),
        validator=attr.validators.optional(attr.validators.instance_of(CertificateTransparencyLogType)),
        metadata={'human_friendly': False},
    )
    log_state = attr.ib(
        default=None,
        converter=convert_dict_to_object(CertificateTransparencyLogState),
        validator=attr.validators.optional(attr.validators.instance_of(CertificateTransparencyLogState))
    )

    def __str__(self):
        return f'{self.description} ({self.log_id})'

    @classmethod
    def description_to_enum_item_name(cls, description):
        name = name_to_enum_item_name(description)

        name = re.sub('([^_])(20[12][0-9][_0-9]*)(H[1-2])?(_LOG)?$', '\\1_\\2\\3\\4', name)

        return name

    @property
    def identifier(self):
        return self.description_to_enum_item_name(self.description)


class CertificateTransparencyLogBase(CryptoDataEnumBase):
    @classmethod
    def from_log_id(cls, log_id):
        log_id = convert_base64_data()(log_id)
        try:
            log = cls._from_attr('log_id', log_id).value
        except InvalidValue:
            log = CertificateTransparencyLogUnknown(log_id)

        return log


CertificateTransparencyLog = CertificateTransparencyLogBase(
    'CertificateTransparencyLog', CryptoDataEnumBase.get_json_records(CertificateTransparencyLogParams)
)


RootCertificateTrustConstraintAction = enum.Enum('RootCertificateTrustConstraintAction', 'DISTRUST DISTURB')


@attr.s(frozen=True)
class CertificateTrustConstraint():
    action = attr.ib(
        converter=convert_enum(RootCertificateTrustConstraintAction),
        validator=attr.validators.instance_of(RootCertificateTrustConstraintAction),
    )
    domains = attr.ib(
        default=(),
        validator=attr.validators.deep_iterable(attr.validators.instance_of(str))
    )
    date = attr.ib(
        default=None,
        converter=convert_datetime('%Y-%m-%dT%H:%M:%S'),
        validator=attr.validators.optional(attr.validators.instance_of(datetime.datetime))
    )


@attr.s(repr=False, slots=True, hash=True)
class _RootCertificateParamCertificateConverter():
    def __call__(self, value):
        if value is None:
            return None

        if isinstance(value, PublicKeyX509Base):
            return value

        if not isinstance(value, (list, tuple)):
            return value

        try:
            return PublicKeyX509Base.from_pem_lines(value)
        except ValueError:
            pass

        return value

    def __repr__(self):
        return '<root certificate parameter converter>'


def convert_root_certificate_params():
    return _RootCertificateParamCertificateConverter()


@attr.s(frozen=True)
class RootCertificateTrustStoreConstraint(CryptoDataParamsBase):
    owner = attr.ib(
        converter=convert_enum(Entity),
        validator=attr.validators.instance_of(Entity)
    )
    constraints = attr.ib(
        default=(),
        converter=convert_iterable(convert_dict_to_object(CertificateTrustConstraint)),
        validator=attr.validators.deep_iterable(attr.validators.instance_of(CertificateTrustConstraint))
    )


@attr.s(frozen=True)
class RootCertificateParams(CryptoDataParamsFetchedBase):
    certificate = attr.ib(
        converter=convert_root_certificate_params(),
        validator=attr.validators.instance_of(PublicKeyX509Base)
    )
    trust_stores = attr.ib(
        default=(),
        converter=convert_iterable(convert_dict_to_object(RootCertificateTrustStoreConstraint)),
        validator=attr.validators.deep_iterable(attr.validators.instance_of(RootCertificateTrustStoreConstraint))
    )

    @classmethod
    def subject_to_enum_item_name(cls, subject, serial_number):
        if 'common_name' in subject:
            name = subject['common_name']
        elif 'organizational_unit_name' in subject:
            name = subject['organizational_unit_name']
        else:
            name = subject['organization_name']

        if not isinstance(name, str):
            name = ' '.join(name)

        return f'{name_to_enum_item_name(name)}_{serial_number}'

    @property
    def identifier(self):
        return self.subject_to_enum_item_name(self.certificate.subject, self.certificate.serial_number)

    def _asdict(self):
        dict_value = super()._asdict()
        dict_value['certificate'] = self.certificate.pem.splitlines()
        meta = collections.OrderedDict([
            ('_subject', self.certificate.subject),
            ('_fingerprints', collections.OrderedDict([
                (hash_algorithm.name, fingerprint)
                for hash_algorithm, fingerprint in self.certificate.fingerprints.items()
            ])),
        ])

        return collections.OrderedDict([('_meta', meta)] + list(dict_value.items()))

    def get_constraints_by_owner(self, owner):
        for trust_store_constraint in self.trust_stores:
            if trust_store_constraint.owner == owner:
                return trust_store_constraint.constraints

        raise KeyError(owner)


class RootCertificateBase(CryptoDataEnumBase):
    @classmethod
    def get_json_encoding(cls):
        return 'utf-8'

    @classmethod
    def get_item_by_sha2_256_fingerprint(cls, fingerprint_value):
        if not hasattr(cls, '_ITEMS_BY_SHA2_256_HASH'):
            cls._ITEMS_BY_SHA2_256_HASH = {
                bytes_to_hex_string(item.value.certificate.get_digest(Hash.SHA2_256)): item
                for item in cls
            }

        return cls._ITEMS_BY_SHA2_256_HASH[fingerprint_value.upper().replace(':', '')]

    @classmethod
    def get_items_by_trust_owner(cls, trust_owner):
        if not hasattr(cls, '_ITEMS_BY_TRUST_OWNER'):
            cls._ITEMS_BY_TRUST_OWNER = {
                _trust_owner: []
                for _trust_owner in Entity.get_items_by_role(EntityRole.CA_TRUST_STORE_OWNER)
            }

            for root_certificate in cls:
                for _trust_owner in map(lambda trust_store: trust_store.owner, root_certificate.value.trust_stores):
                    cls._ITEMS_BY_TRUST_OWNER[_trust_owner].append(root_certificate)

            cls._ITEMS_BY_TRUST_OWNER = cls._ITEMS_BY_TRUST_OWNER

        return cls._ITEMS_BY_TRUST_OWNER[trust_owner]


RootCertificate = RootCertificateBase(
    'RootCertificate', RootCertificateBase.get_json_records(RootCertificateParams)
)



# -*- coding: utf-8 -*-

import collections
import datetime
import pathlib
import unittest

from test.common.classes import (
    TestEnumNamedParams,
    TestEnumNumericParams,
    TestEnumOidParams,
    TestEnumStringParams,
    TestJsonObject,
    TestJsonObjectComplex,
    EnumIntTest,
    EnumTest,
)

import dateutil
import pyfakefs.fake_filesystem_unittest
import urllib3

from cryptodatahub.common.exception import InvalidValue
from cryptodatahub.common.types import (
    Base64Data,
    ClientVersion,
    CryptoDataEnumBase,
    CryptoDataEnumCodedBase,
    CryptoDataEnumOIDBase,
    convert_big_enum,
    convert_base64_data,
    convert_client_version,
    convert_datetime,
    convert_dict_to_object,
    convert_enum,
    convert_iterable,
    convert_mapping,
    convert_url,
    convert_variadic,
    convert_value_to_object,
)


class TestEnumConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1234
        converted_value = convert_enum(EnumTest)(original_value)
        self.assertEqual(id(original_value), id(convert_enum(EnumTest)(converted_value)))

    def test_error_invalid_value(self):
        original_value = 'not in enum'
        converted_value = convert_enum(EnumTest)(original_value)
        self.assertEqual(id(converted_value), id(convert_enum(EnumTest)(converted_value)))

    def test_none(self):
        converted_value = convert_enum(EnumTest)(None)
        self.assertEqual(converted_value, None)

    def test_convert(self):
        original_value = 'STRING'
        converted_value = convert_enum(EnumTest)(original_value)

        self.assertEqual(converted_value, EnumTest.STRING)

    def test_repr(self):
        self.assertEqual(repr(convert_enum(EnumTest)), '<enum converter>')


class TestDateTimeConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1
        converted_value = convert_datetime(None)(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_error_invalid_value(self):
        original_value = 'not a date'
        converted_value = convert_datetime(None)(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_none(self):
        converted_value = convert_datetime(None)(None)
        self.assertEqual(converted_value, None)

    def test_convert(self):
        original_value = '2023-04-15T20:01:02+00:00'
        converted_value = convert_datetime(None)(original_value)

        self.assertEqual(converted_value, datetime.datetime(2023, 4, 15, 20, 1, 2, tzinfo=dateutil.tz.UTC))
        self.assertEqual(id(converted_value), id(convert_datetime(None)(converted_value)))

        original_value = '21/11/06 16:30'
        converted_value = convert_datetime('%d/%m/%y %H:%M')(original_value)

        self.assertEqual(converted_value,  datetime.datetime(2006, 11, 21, 16, 30))
        self.assertEqual(id(converted_value), id(convert_datetime(None)(converted_value)))

    def test_repr(self):
        self.assertEqual(repr(convert_datetime(None)), '<datetime converter>')


class TestBase64Data(unittest.TestCase):
    def test_str(self):
        self.assertEqual(str(Base64Data(b'light work.')), 'bGlnaHQgd29yay4=')

    def test_asdict(self):
        base64_data = Base64Data(b'light work.')
        self.assertEqual(str(base64_data), base64_data._asdict())


class TestBase64DataConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 'not a binary value'
        converted_value = convert_base64_data()(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_error_invalid_value(self):
        original_value = 'not a base64 str'
        converted_value = convert_base64_data()(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_error_invalid_padding(self):
        original_value = 'invalid padding'
        converted_value = convert_base64_data()(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_none(self):
        converted_value = convert_base64_data()(None)
        self.assertEqual(converted_value, None)

    def test_binary_value(self):
        original_value = bytearray(b'binary value')
        converted_value = convert_base64_data()(original_value)
        self.assertNotEqual(id(original_value), id(converted_value))

    def test_convert(self):
        original_value = 'bGlnaHQgd29yay4='
        converted_value = convert_base64_data()(original_value)

        self.assertEqual(converted_value, Base64Data(b'light work.'))
        self.assertEqual(id(converted_value), id(convert_base64_data()(converted_value)))

        original_value = 'bGlnaHQgd29yay4=='
        converted_value = convert_base64_data()(original_value)

        self.assertEqual(converted_value, Base64Data(b'light work.'))
        self.assertEqual(id(converted_value), id(convert_base64_data()(converted_value)))

    def test_repr(self):
        self.assertEqual(repr(convert_base64_data()), '<base64 data converter>')


class TestDictToObjectConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1234
        converted_value = convert_dict_to_object(TestJsonObject)(original_value)
        self.assertEqual(original_value, converted_value)

    def test_error_invalid_value(self):
        original_value = 1.0
        converted_value = convert_dict_to_object(TestJsonObject)(original_value)
        self.assertEqual(original_value, converted_value)

    def test_none(self):
        converted_value = convert_dict_to_object(TestJsonObject)(None)
        self.assertEqual(converted_value, None)

    def test_convert(self):
        original_value = {
            'attr_simple': 1,
            'attr_iterable': [1, 2, 3, 4],
            'attr_complex': {'attr': 'value'},
        }

        converted_value = convert_dict_to_object(TestJsonObject)(original_value)

        self.assertEqual(
            converted_value,
            TestJsonObject(
                attr_simple=1,
                attr_iterable=[1, 2, 3, 4],
                attr_complex=TestJsonObjectComplex('value'),
            )
        )
        self.assertEqual(id(converted_value), id(convert_dict_to_object(str)(converted_value)))

    def test_asdict(self):
        self.assertEqual(
            TestJsonObject(
                attr_simple=1,
                attr_iterable=[1, 2, 3, 4],
                attr_complex=TestJsonObjectComplex('value'),
            )._asdict(),
            collections.OrderedDict([
                ('attr_simple', 1),
                ('attr_iterable', [1, 2, 3, 4]),
                ('attr_complex', {'attr': 'value'}),
            ])
        )

    def test_repr(self):
        self.assertEqual(repr(convert_dict_to_object(str)), '<dict to object converter>')


class TestValueToObjectConverter(unittest.TestCase):
    def test_none(self):
        converted_value = convert_value_to_object(TestJsonObjectComplex)(None)
        self.assertEqual(converted_value, None)

    def test_convert_object_class(self):
        original_value = TestJsonObjectComplex('value')
        converted_value = convert_value_to_object(TestJsonObjectComplex)(original_value)
        self.assertEqual(original_value, converted_value)

    def test_convert_converter(self):
        original_value = '1970-01-01'
        converted_value = convert_value_to_object(TestJsonObjectComplex, convert_datetime())(original_value)
        self.assertEqual(converted_value.attr, datetime.datetime(1970, 1, 1, 0, 0))

    def test_repr(self):
        self.assertEqual(repr(convert_value_to_object(str)), '<value to object converter>')


class TestBigNumberConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1234
        converted_value = convert_big_enum()(original_value)
        self.assertEqual(id(original_value), id(convert_big_enum()(converted_value)))

    def test_error_invalid_value(self):
        original_value = 'not a big number'
        converted_value = convert_big_enum()(original_value)
        self.assertEqual(id(converted_value), id(convert_big_enum()(converted_value)))

        original_value = [1, 2, 3, 4]
        converted_value = convert_big_enum()(original_value)
        self.assertEqual(id(converted_value), id(convert_big_enum()(converted_value)))

    def test_none(self):
        converted_value = convert_big_enum()(None)
        self.assertEqual(converted_value, None)

    def test_convert(self):
        # C code-style
        original_value = [
            '0x01, 0x23',
            '0x45, 0x67'
        ]
        converted_value = convert_big_enum()(original_value)
        self.assertEqual(converted_value, 0x1234567)

        # RFC-style
        original_value = [
            '01234567 89abcdef',
            '89abcdef 01234567',
        ]
        converted_value = convert_big_enum()(original_value)
        self.assertEqual(converted_value, 0x0123456789abcdef89abcdef01234567)

    def test_repr(self):
        self.assertEqual(repr(convert_big_enum()), '<big number converter>')


class TestIterableConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1234
        converted_value = convert_iterable(str)(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_error_invalid_value(self):
        original_value = 1.0
        converted_value = convert_iterable(str)(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_none(self):
        converted_value = convert_iterable(str)(None)
        self.assertEqual(converted_value, None)

    def test_convert(self):
        original_value = [1, 2, 3]
        converted_value = convert_iterable(str)(original_value)

        self.assertEqual(converted_value, ['1', '2', '3'])
        self.assertEqual(id(converted_value), id(convert_iterable(str)(converted_value)))

    def test_repr(self):
        self.assertEqual(repr(convert_iterable(str)), '<iterable converter>')


class TestMappingConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1234
        converted_value = convert_mapping(str)(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_error_invalid_value_key(self):
        original_value = {'not a number': 1}
        converted_value = convert_mapping(key_converter=int)(original_value)
        self.assertEqual(original_value, converted_value)

    def test_error_invalid_value_value(self):
        original_value = {1: 'not a number'}
        converted_value = convert_mapping(key_converter=int)(original_value)
        self.assertEqual(original_value, converted_value)

    def test_none(self):
        converted_value = convert_mapping(str)(None)
        self.assertEqual(converted_value, None)

    def test_convert_key(self):
        original_value = {1: '1', 2: '2', 3: '3'}
        converted_value = convert_mapping(key_converter=str)(original_value)

        self.assertEqual(converted_value, {'1': '1', '2': '2', '3': '3'})
        self.assertNotEqual(id(converted_value), id(convert_mapping(str)(converted_value)))

    def test_convert_value(self):
        original_value = {'1': 1, '2': 2, '3': 3}
        converted_value = convert_mapping(value_converter=str)(original_value)

        self.assertEqual(converted_value, {'1': '1', '2': '2', '3': '3'})
        self.assertNotEqual(id(converted_value), id(convert_mapping(str)(converted_value)))

    def test_repr(self):
        self.assertEqual(repr(convert_mapping(str)), '<mapping converter>')


class TestUrlConverter(unittest.TestCase):
    def test_error_invalid_value(self):
        original_value = 'https://example.com:123456'
        converted_value = convert_url()(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_none(self):
        converted_value = convert_url()(None)
        self.assertEqual(converted_value, None)

    def test_convert_value(self):
        original_value = 'https://example.com'
        converted_value = convert_url()(original_value)

        self.assertEqual(
            converted_value,
            urllib3.util.url.Url(scheme='https', host='example.com')
        )
        self.assertEqual(id(converted_value), id(convert_url()(converted_value)))

    def test_repr(self):
        self.assertEqual(repr(convert_url()), '<url converter>')


class TestClientVariadicConverter(unittest.TestCase):
    def test_repr(self):
        self.assertEqual(repr(convert_variadic([])), '<variadic converter>')

    def test_error_invalid_value(self):
        original_value = 'non-enum'
        converted_value = convert_variadic([convert_enum(EnumIntTest)])(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_none(self):
        converted_value = convert_variadic([])(None)
        self.assertEqual(converted_value, None)

    def test_convert_value(self):
        original_value = 'ONE'
        converted_value = convert_variadic([convert_enum(EnumIntTest)])(original_value)

        self.assertEqual(converted_value, EnumIntTest.ONE)
        self.assertEqual(id(converted_value), id(convert_variadic([convert_enum(EnumIntTest)])(converted_value)))

        original_value = 'NONE'
        converted_value = convert_variadic([convert_enum(EnumIntTest), convert_enum(EnumTest)])(original_value)

        self.assertEqual(converted_value, EnumTest.NONE)
        self.assertEqual(
            id(converted_value),
            id(convert_variadic([convert_enum(EnumIntTest), convert_enum(EnumTest)])(converted_value))
        )


class TestClientVersionConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1234
        converted_value = convert_client_version()(original_value)
        self.assertEqual(id(original_value), id(converted_value))

    def test_error_invalid_value(self):
        original_value = 'neither a single not a dotted number'
        converted_value = convert_client_version()(original_value)
        self.assertEqual(original_value, converted_value)

    def test_none(self):
        converted_value = convert_client_version()(None)
        self.assertEqual(converted_value, None)

    def test_convert_value(self):
        original_value = '1'
        converted_value = convert_client_version()(original_value)

        self.assertEqual(converted_value, ClientVersion((1, )))
        self.assertEqual(id(converted_value), id(convert_client_version()(converted_value)))

        original_value = '1.2.3'
        converted_value = convert_client_version()(original_value)

        self.assertEqual(converted_value, ClientVersion((1, 2, 3)))
        self.assertEqual(id(converted_value), id(convert_client_version()(converted_value)))

    def test_repr(self):
        self.assertEqual(repr(convert_client_version()), '<client version converter>')


class TestCryptoDataBase(pyfakefs.fake_filesystem_unittest.TestCase):
    def setUp(self):  # pylint: disable=invalid-name
        self.setUpPyfakefs()

    def _set_json(self, param_class, json_object):
        self.fs.create_dir(str(CryptoDataEnumBase.get_json_path(param_class).parent))
        CryptoDataEnumBase.set_json(param_class, json_object)


class TestCryptoDataEnumNamed(TestCryptoDataBase):
    def test_str(self):
        self._set_json(
            TestEnumNamedParams,
            {'NAME': {'name': 'name', 'long_name': 'long_name'}}
        )
        test_enum_named_class = CryptoDataEnumBase(
            'test_enum_named_class', CryptoDataEnumBase.get_json_records(TestEnumNamedParams)
        )
        self.assertEqual(str(test_enum_named_class.NAME.value), 'name')


class TestCryptoDataEnumBase(TestCryptoDataBase):
    def test_error_get_json_path(self):
        with self.assertRaises(TypeError) as context_manager:
            self._set_json(TestCryptoDataEnumBase, {})
        self.assertEqual(context_manager.exception.args, (TestCryptoDataEnumBase, ))

    def test_get_json_path(self):
        json_path = CryptoDataEnumBase.get_json_path(TestEnumNamedParams)
        self.assertTrue(isinstance(json_path, pathlib.Path))
        self.assertEqual(json_path.name, 'test-enum-named.json')

    def test_get_json(self):
        json_object = {
            'ONE': {
                'code': 1
            },
            'TWO': {
                'code': 2
            }
        }
        self._set_json(TestEnumNumericParams, json_object)
        self.assertEqual(CryptoDataEnumBase.get_json(TestEnumNumericParams), json_object)

    def test_get_records(self):
        json_object = {
            'NAME': {
                '_hidden_key_1': 'value',
                'name': 'name',
                '_hidden_key_2': 'value',
                'long-name': 'long-name'
            }
        }
        self._set_json(TestEnumNamedParams, json_object)
        json_records = CryptoDataEnumBase.get_json_records(TestEnumNamedParams)
        self.assertEqual(json_records, collections.OrderedDict([
             ('NAME', TestEnumNamedParams(name='name', long_name='long-name')),
        ]))


class TestCryptoDataEnumCodedBase(TestCryptoDataBase):
    def test_from_code(self):
        json_object = {
             'ONE': {
                 'code': 1
             },
             'TWO': {
                 'code': 2
             }
         }
        self._set_json(TestEnumNumericParams, json_object)
        test_enum_numeric_class = CryptoDataEnumCodedBase(
            'test_enum_numeric_class', CryptoDataEnumCodedBase.get_json_records(TestEnumNumericParams)
        )
        self.assertEqual(test_enum_numeric_class.from_code(1), test_enum_numeric_class.ONE)
        self.assertEqual(test_enum_numeric_class.from_code(2), test_enum_numeric_class.TWO)
        with self.assertRaises(InvalidValue) as context_manager:
            test_enum_numeric_class.from_code(3)
        self.assertEqual(context_manager.exception.value, 3)


class TestCryptoDataEnumOidBase(TestCryptoDataBase):
    def test_from_oid(self):
        json_object = {
            'ONE': {
                'name': 'one',
                'long_name': None,
                'oid': '1.1.1.1.1.1'
            },
            'TWO': {
                'name': 'two',
                'long_name': None,
                'oid': '2.2.2.2.2.2'
            }
        }
        self._set_json(TestEnumOidParams, json_object)
        test_enum_oid_class = CryptoDataEnumOIDBase(
            'test_enum_oid_class', CryptoDataEnumCodedBase.get_json_records(TestEnumOidParams)
        )
        self.assertEqual(test_enum_oid_class.from_oid('1.1.1.1.1.1'), test_enum_oid_class.ONE)
        self.assertEqual(test_enum_oid_class.from_oid('2.2.2.2.2.2'), test_enum_oid_class.TWO)


class TestCryptoDataEnumNumeric(TestCryptoDataBase):
    def test_error_code_negativ(self):
        self._set_json(TestEnumNumericParams, {'NAME': {'code': -1}})
        with self.assertRaises(ValueError) as context_manager:
            CryptoDataEnumBase(
                'TestEnumNumeric', CryptoDataEnumBase.get_json_records(TestEnumNumericParams)
            )
        self.assertEqual(context_manager.exception.args, (-1, ))

    def test_error_code_too_large(self):
        self._set_json(TestEnumNumericParams, {'NAME': {'code': 256}})
        with self.assertRaises(ValueError) as context_manager:
            CryptoDataEnumBase(
                'TestEnumNumeric', CryptoDataEnumBase.get_json_records(TestEnumNumericParams)
            )
        self.assertEqual(context_manager.exception.args, (256, ))

    def test_error_non_numeric(self):
        self._set_json(TestEnumNumericParams, {'NAME': {'code': 'non-numeric'}})
        with self.assertRaises(ValueError) as context_manager:
            CryptoDataEnumBase(
                'TestEnumNumeric', CryptoDataEnumBase.get_json_records(TestEnumNumericParams)
            )
        self.assertEqual(context_manager.exception.args, ('non-numeric', ))


class TestCryptoDataEnumString(TestCryptoDataBase):
    def test_str(self):
        self._set_json(TestEnumStringParams, {'NAME': {'code': 'string'}})
        test_enum_string_class = CryptoDataEnumBase(
            'test_enum_string_class', CryptoDataEnumBase.get_json_records(TestEnumStringParams)
        )

        self.assertEqual(str(test_enum_string_class.NAME.value), 'string')

    def test_asdict(self):
        self._set_json(TestEnumStringParams, {'NAME': {'code': 'dict'}})
        test_enum_string_class = CryptoDataEnumBase(
            'test_enum_string_class', CryptoDataEnumBase.get_json_records(TestEnumStringParams)
        )

        self.assertEqual(test_enum_string_class.NAME.value._asdict(), 'dict')

    def test_get_code_size(self):
        self._set_json(TestEnumStringParams, {'NAME': {'code': 'four'}})
        test_enum_string_class = CryptoDataEnumBase(
            'test_enum_string_class', CryptoDataEnumBase.get_json_records(TestEnumStringParams)
        )

        self.assertEqual(test_enum_string_class.NAME.value.get_code_size(), 4)



# -*- coding: utf-8 -*-

import collections
import unittest

from test.common.classes import TestClasses


from cryptodatahub.common.algorithm import Hash
from cryptodatahub.common.entity import Entity
from cryptodatahub.common.key import PublicKeyX509Base
from cryptodatahub.common.stores import (
    CertificateTransparencyLog,
    CertificateTransparencyLogParams,
    CertificateTransparencyLogUnknown,
    RootCertificate,
    RootCertificateParams,
    RootCertificateTrustStoreConstraint,
    convert_root_certificate_params,
)


class TestCertificateTransparencyLog(TestClasses.TestJsonBase):
    @classmethod
    def _get_class(cls):
        return CertificateTransparencyLog

    def test_error_mmd_too_small(self):
        with self.assertRaises(ValueError) as context_manager:
            CertificateTransparencyLogParams(
                operator=Entity.GOOGLE,
                log_id='=',
                key=b'',
                url='',
                mmd=0
            )
        self.assertEqual(context_manager.exception.args, ('\'mmd\' must be >= 1: 0',))

    def test_from_log_id(self):
        self.assertEqual(
            CertificateTransparencyLog.from_log_id('lgbALGkAM6odFF9ZxuJkjQVJ8N+WqrjbkVpw2OzzkKU='),
            CertificateTransparencyLog.AKAMAI_CT_LOG.value
        )
        self.assertEqual(
            CertificateTransparencyLog.from_log_id(bytearray(b'unkown log')),
            CertificateTransparencyLogUnknown(bytearray(b'unkown log'))
        )

    def test_convert_description(self):
        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('CaMeL CaSe'),
            'CAMEL_CASE'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('S"p+e!c%i/a=l'),
            'S_P_E_C_I_A_L'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('S""p++e!!c%%i//a==l'),
            'S_P_E_C_I_A_L'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('  leading spaces'),
            'LEADING_SPACES'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('trailing spaces  '),
            'TRAILING_SPACES'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('Organization Log'),
            'ORGANIZATION_LOG'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('Organization Log 2'),
            'ORGANIZATION_LOG_2'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('Organization Log 2023'),
            'ORGANIZATION_LOG_2023'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('Organization Log 2023 2'),
            'ORGANIZATION_LOG_2023_2'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('Organization Log 2023H2'),
            'ORGANIZATION_LOG_2023H2'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('Organization Log v1 2023'),
            'ORGANIZATION_LOG_V1_2023'
        )

        self.assertEqual(
            CertificateTransparencyLogParams.description_to_enum_item_name('Organization Ct Log 2023'),
            'ORGANIZATION_CT_LOG_2023'
        )

    def test_str(self):
        self.assertEqual(
            str(CertificateTransparencyLog.from_log_id(bytearray(b'unknown log'))),
            'dW5rbm93biBsb2c='
        )
        self.assertEqual(
            str(CertificateTransparencyLog.AKAMAI_CT_LOG.value),
            'Akamai CT Log (lgbALGkAM6odFF9ZxuJkjQVJ8N+WqrjbkVpw2OzzkKU=)'
        )


class TestRootCertificateParams(TestClasses.TestKeyBase):
    def test_subject_to_enum_item_name(self):
        self.assertEqual(
            RootCertificateParams.subject_to_enum_item_name(
                {'common_name': 'common name'},
                1
            ),
            'COMMON_NAME_1'
        )

        self.assertEqual(
            RootCertificateParams.subject_to_enum_item_name(
                {'common_name': 'common name',
                 'organizational_unit_name': 'organizational unit'},
                1
            ),
            'COMMON_NAME_1'
        )

        self.assertEqual(
            RootCertificateParams.subject_to_enum_item_name(
                {'common_name': 'common name',
                 'organization_name': 'organization'},
                1
            ),
            'COMMON_NAME_1'
        )

        self.assertEqual(
            RootCertificateParams.subject_to_enum_item_name(
                {'organizational_unit_name': 'organizational unit'},
                1
            ),
            'ORGANIZATIONAL_UNIT_1'
        )

        self.assertEqual(
            RootCertificateParams.subject_to_enum_item_name(
                {'organizational_unit_name': 'organizational unit',
                 'organization_name': 'organization'},
                1
            ),
            'ORGANIZATIONAL_UNIT_1'
        )

        self.assertEqual(
            RootCertificateParams.subject_to_enum_item_name(
                {'organization_name': 'organization'},
                1
            ),
            'ORGANIZATION_1'
        )

        self.assertEqual(
            RootCertificateParams.subject_to_enum_item_name(
                {'common_name': ('common',
                                 'name')},
                1
            ),
            'COMMON_NAME_1'
        )

    def test_identifier(self):
        self.assertEqual(
            RootCertificateParams(self._get_public_key_x509('snakeoil_ca_cert')).identifier,
            'DEFAULT_COMPANY_CA_551843783816852671294343937890484223348994332352'
        )

    def test_asdict(self):
        self.assertEqual(
            RootCertificateParams(self._get_public_key_x509('snakeoil_ca_cert'))._asdict(),
            collections.OrderedDict([
                ('_meta', collections.OrderedDict([
                    ('_subject', collections.OrderedDict([
                        ('country_name', 'XX'),
                        ('locality_name', 'Default City'),
                        ('organization_name', 'Default Company Ltd'),
                        ('common_name', 'Default Company CA')
                    ])),
                    ('_fingerprints', collections.OrderedDict([
                        ('MD5',
                         'D5:34:BF:D6:13:2A:75:9C:A8:3D:AD:AC:CE:B3:9E:84'),
                        ('SHA1',
                         'F8:26:7B:11:7C:85:C7:EB:C6:0F:01:E2:66:2A:82:C9:42:12:4A:62'),
                        ('SHA2_256',
                         '8D:45:A1:27:4C:B4:51:AB:02:A6:37:C3:63:62:AF:65:'
                         '30:0F:A5:54:44:A6:E7:60:35:A7:45:79:E8:64:B9:D9')
                    ]))
                ])),
                ('certificate', [
                    '-----BEGIN CERTIFICATE-----',
                    'MIIDnzCCAoegAwIBAgIUYKmH9ghjSXftrJhqTq2zfLUJ1sAwDQYJKoZIhvcNAQEL',
                    'BQAwXzELMAkGA1UEBhMCWFgxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEcMBoGA1UE',
                    'CgwTRGVmYXVsdCBDb21wYW55IEx0ZDEbMBkGA1UEAwwSRGVmYXVsdCBDb21wYW55',
                    'IENBMB4XDTIzMDUyOTE3Mjc0OVoXDTI4MDUyNzE3Mjc0OVowXzELMAkGA1UEBhMC',
                    'WFgxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEcMBoGA1UECgwTRGVmYXVsdCBDb21w',
                    'YW55IEx0ZDEbMBkGA1UEAwwSRGVmYXVsdCBDb21wYW55IENBMIIBIjANBgkqhkiG',
                    '9w0BAQEFAAOCAQ8AMIIBCgKCAQEAueVK46q1gO6TRgDsSKJ1VHZO4GAMrholea/2',
                    'RR832DSRgI9wKC3KVrrc+RTev5l5JWlor8cctOLipxT8XsSeyH8S30pU38xkCyPL',
                    'zFVm7abJ1GXZcRQ/bJ241mgc8D4ugvsbsUPNX7ff1D/dGsSh4Rjpco2lEzw0kLFp',
                    'NmKkc2h1UUApSktWurewVFdcQ8I9u9OqD3tc0KeSMejzVBOkctcYChVAZRoTfXKV',
                    '9pvKIayFJN739zmJSWS2pxPNCgKp6sYMvMPpbU9AiTQYa1zenon/k6A94Gn3yWxG',
                    'tBpTN+vInTLZe1OmM9uXwOkFWjWbfaGYcOIrFGhhaMxSfOVeMwIDAQABo1MwUTAd',
                    'BgNVHQ4EFgQUx3cgEXkwESXclHhsdJWU63Vv8kUwHwYDVR0jBBgwFoAUx3cgEXkw',
                    'ESXclHhsdJWU63Vv8kUwDwYDVR0TAQH/BAUwAwEB/zANBgkqhkiG9w0BAQsFAAOC',
                    'AQEAibNgxyAIUurv1YCIsD9SmfeQTIQMT80d2K+Wc0u0tk5eVACV3HecmLz7JNU0',
                    'yPPkUVnG2/B93ZPpaYGuRFPC+r32ZZYZABRcgAJIVBqtCYxq8ZWY/vbY5F6SWdnJ',
                    'MsWhVgrjqmRNRQkhPNf/LJ8fvXk+BROODrtfYPDsHoVteaYQkdDrnahlbF5HhxeO',
                    'Iskcr9tFo4oUsZpUIOYiY3go14PfZuq1h11ULD1qf7e1H+Fk2knl820wFrgc6BTx',
                    'j/ftJD5c/0T7cnVov5veG0W6sU1MAYop8g/2Xc4eMtRfamiU5scLzpvl1OdxGExw',
                    'Ek966UKB55AyKxqVAQBRb3yeCw==',
                    '-----END CERTIFICATE-----'
                ]),
            ])
        )

    def test_get_constraints_by_owner(self):
        root_certificate_param = RootCertificateParams(RootCertificate.AAA_CERTIFICATE_SERVICES_1.value.certificate)
        with self.assertRaises(KeyError) as context_manager:
            root_certificate_param.get_constraints_by_owner(Entity.APPLE)
        self.assertEqual(context_manager.exception.args, (Entity.APPLE, ))

        root_certificate_param = RootCertificateParams(
            RootCertificate.AAA_CERTIFICATE_SERVICES_1.value.certificate,
            [RootCertificateTrustStoreConstraint(Entity.GOOGLE)]
        )
        with self.assertRaises(KeyError) as context_manager:
            root_certificate_param.get_constraints_by_owner(Entity.APPLE)
        self.assertEqual(context_manager.exception.args, (Entity.APPLE, ))

        root_certificate_param = RootCertificateParams(
            RootCertificate.AAA_CERTIFICATE_SERVICES_1.value.certificate,
            [RootCertificateTrustStoreConstraint(Entity.APPLE)]
        )
        self.assertEqual(root_certificate_param.get_constraints_by_owner(Entity.APPLE), ())


class TestRootCertificateParamCertificateConverter(unittest.TestCase):
    def test_error_invalid_type(self):
        original_value = 1234
        converted_value = convert_root_certificate_params()(original_value)
        self.assertEqual(id(original_value), id(convert_root_certificate_params()(converted_value)))

    def test_error_invalid_value(self):
        original_value = ()
        converted_value = convert_root_certificate_params()(original_value)
        self.assertEqual(id(converted_value), id(convert_root_certificate_params()(converted_value)))

    def test_none(self):
        converted_value = convert_root_certificate_params()(None)
        self.assertEqual(converted_value, None)

    def test_convert(self):
        original_value = [
            '-----BEGIN CERTIFICATE-----',
            'MIIDnzCCAoegAwIBAgIUYKmH9ghjSXftrJhqTq2zfLUJ1sAwDQYJKoZIhvcNAQEL',
            'BQAwXzELMAkGA1UEBhMCWFgxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEcMBoGA1UE',
            'CgwTRGVmYXVsdCBDb21wYW55IEx0ZDEbMBkGA1UEAwwSRGVmYXVsdCBDb21wYW55',
            'IENBMB4XDTIzMDUyOTE3Mjc0OVoXDTI4MDUyNzE3Mjc0OVowXzELMAkGA1UEBhMC',
            'WFgxFTATBgNVBAcMDERlZmF1bHQgQ2l0eTEcMBoGA1UECgwTRGVmYXVsdCBDb21w',
            'YW55IEx0ZDEbMBkGA1UEAwwSRGVmYXVsdCBDb21wYW55IENBMIIBIjANBgkqhkiG',
            '9w0BAQEFAAOCAQ8AMIIBCgKCAQEAueVK46q1gO6TRgDsSKJ1VHZO4GAMrholea/2',
            'RR832DSRgI9wKC3KVrrc+RTev5l5JWlor8cctOLipxT8XsSeyH8S30pU38xkCyPL',
            'zFVm7abJ1GXZcRQ/bJ241mgc8D4ugvsbsUPNX7ff1D/dGsSh4Rjpco2lEzw0kLFp',
            'NmKkc2h1UUApSktWurewVFdcQ8I9u9OqD3tc0KeSMejzVBOkctcYChVAZRoTfXKV',
            '9pvKIayFJN739zmJSWS2pxPNCgKp6sYMvMPpbU9AiTQYa1zenon/k6A94Gn3yWxG',
            'tBpTN+vInTLZe1OmM9uXwOkFWjWbfaGYcOIrFGhhaMxSfOVeMwIDAQABo1MwUTAd',
            'BgNVHQ4EFgQUx3cgEXkwESXclHhsdJWU63Vv8kUwHwYDVR0jBBgwFoAUx3cgEXkw',
            'ESXclHhsdJWU63Vv8kUwDwYDVR0TAQH/BAUwAwEB/zANBgkqhkiG9w0BAQsFAAOC',
            'AQEAibNgxyAIUurv1YCIsD9SmfeQTIQMT80d2K+Wc0u0tk5eVACV3HecmLz7JNU0',
            'yPPkUVnG2/B93ZPpaYGuRFPC+r32ZZYZABRcgAJIVBqtCYxq8ZWY/vbY5F6SWdnJ',
            'MsWhVgrjqmRNRQkhPNf/LJ8fvXk+BROODrtfYPDsHoVteaYQkdDrnahlbF5HhxeO',
            'Iskcr9tFo4oUsZpUIOYiY3go14PfZuq1h11ULD1qf7e1H+Fk2knl820wFrgc6BTx',
            'j/ftJD5c/0T7cnVov5veG0W6sU1MAYop8g/2Xc4eMtRfamiU5scLzpvl1OdxGExw',
            'Ek966UKB55AyKxqVAQBRb3yeCw==',
            '-----END CERTIFICATE-----',
        ]
        converted_value = convert_root_certificate_params()(original_value)
        self.assertTrue(isinstance(converted_value, PublicKeyX509Base))
        self.assertEqual(id(converted_value), id(convert_root_certificate_params()(converted_value)))

    def test_repr(self):
        self.assertEqual(repr(convert_root_certificate_params()), '<root certificate parameter converter>')


class TestRootCertificate(unittest.TestCase):
    def test_get_items_by_trust_owner(self):
        root_certificate_count_by_owner = collections.OrderedDict([
            (Entity.MOZILLA, 141),
            (Entity.MICROSOFT, 299),
            (Entity.APPLE, 172),
            (Entity.GOOGLE, 134),
        ])

        for owner, certificate_count in root_certificate_count_by_owner.items():
            with self.subTest(owner=owner):
                root_certificates = RootCertificate.get_items_by_trust_owner(owner)
                self.assertEqual(len(root_certificates), certificate_count)
                self.assertEqual(id(root_certificates), id(RootCertificate.get_items_by_trust_owner(owner)))

    def test_get_item_by_fingerprint(self):
        root_certificate = RootCertificate.AAA_CERTIFICATE_SERVICES_1.value.certificate
        sha2_256_fingerprint = root_certificate.fingerprints[Hash.SHA2_256]
        self.assertEqual(
            RootCertificate.get_item_by_sha2_256_fingerprint(sha2_256_fingerprint),
            RootCertificate.AAA_CERTIFICATE_SERVICES_1
        )
        self.assertEqual(
            RootCertificate.get_item_by_sha2_256_fingerprint(sha2_256_fingerprint.replace(':',
                                                                                          '')),
            RootCertificate.AAA_CERTIFICATE_SERVICES_1
        )



# -*- coding: utf-8 -*-

from test.common.classes import TestClasses

from cryptodatahub.ssh.algorithm import (
    SshEllipticCurveIdentifier,
    SshEncryptionAlgorithm,
    SshMacAlgorithm,
    SshKexAlgorithm,
    SshHostKeyAlgorithm,
    SshCompressionAlgorithm,
)


class TestSshEncryptionAlgorithm(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return SshEncryptionAlgorithm


class TestSshMacAlgorithm(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return SshMacAlgorithm

    def test_size(self):
        self.assertEqual(SshMacAlgorithm.HMAC_SHA2_256.value.size, 256)
        self.assertEqual(SshMacAlgorithm.HMAC_SHA2_256_96.value.size, 96)


class TestSshKexAlgorithm(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return SshKexAlgorithm


class TestSshHostKeyAlgorithm(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return SshHostKeyAlgorithm


class TestSshCompressionAlgorithm(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return SshCompressionAlgorithm


class TestSshEllipticCurveIdentifier(TestClasses.TestJsonCodeStringBase):
    @classmethod
    def _get_class(cls):
        return SshEllipticCurveIdentifier
